==> parsita-2.0.0b1.tar.gz: parsita-2.0.0b1/src/parsita/parsers/_sequential.py <==
__all__ = ["SequentialParser", "DiscardLeftParser", "DiscardRightParser"]

from typing import Any, Generic, List

from ..state import Continue, Input, Output, Reader, State
from ._base import Parser


class SequentialParser(Generic[Input], Parser[Input, List[Any]]):  # Type of this class is inexpressible
    def __init__(self, parser: Parser[Input, Any], *parsers: Parser[Input, Any]):
        super().__init__()
        self.parsers = (parser, *parsers)

    def consume(self, state: State[Input], reader: Reader[Input]):
        output = []
        remainder = reader

        for parser in self.parsers:
            status = parser.cached_consume(state, remainder)
            if isinstance(status, Continue):
                output.append(status.value)
                remainder = status.remainder
            else:
                return None

        return Continue(remainder, output)

    def __repr__(self):
        names = []
        for parser in self.parsers:
            names.append(parser.name_or_repr())

        return self.name_or_nothing() + " & ".join(names)


class DiscardLeftParser(Generic[Input, Output], Parser[Input, Output]):
    def __init__(self, left: Parser[Input, Any], right: Parser[Input, Output]):
        super().__init__()
        self.left = left
        self.right = right

    def consume(self, state: State[Input], reader: Reader[Input]):
        status = self.left.cached_consume(state, reader)
        if isinstance(status, Continue):
            return self.right.cached_consume(state, status.remainder)
        else:
            return None

    def __repr__(self):
        return self.name_or_nothing() + f"{self.left.name_or_repr()} >> {self.right.name_or_repr()}"


class DiscardRightParser(Generic[Input, Output], Parser[Input, Output]):
    def __init__(self, left: Parser[Input, Output], right: Parser[Input, Any]):
        super().__init__()
        self.left = left
        self.right = right

    def consume(self, state: State[Input], reader: Reader[Input]):
        status1 = self.left.cached_consume(state, reader)
        if isinstance(status1, Continue):
            status2 = self.right.cached_consume(state, status1.remainder)
            if isinstance(status2, Continue):
                return Continue(status2.remainder, status1.value)
            else:
                return None
        else:
            return None

    def __repr__(self):
        return self.name_or_nothing() + f"{self.left.name_or_repr()} << {self.right.name_or_repr()}"

==> parsita-2.1.1.tar.gz: parsita-2.1.1/src/parsita/parsers/_sequential.py <==
__all__ = ["SequentialParser", "DiscardLeftParser", "DiscardRightParser"]

from typing import Any, Generic, List

from ..state import Continue, Input, Output, Reader, State
from ._base import Parser


# Type of this class is inexpressible
class SequentialParser(Generic[Input], Parser[Input, List[Any]]):
    def __init__(self, parser: Parser[Input, Any], *parsers: Parser[Input, Any]):
        super().__init__()
        self.parsers = (parser, *parsers)

    def _consume(self, state: State[Input], reader: Reader[Input]):
        output = []
        remainder = reader

        for parser in self.parsers:
            status = parser.consume(state, remainder)
            if isinstance(status, Continue):
                output.append(status.value)
                remainder = status.remainder
            else:
                return None

        return Continue(remainder, output)

    def __repr__(self):
        names = []
        for parser in self.parsers:
            names.append(parser.name_or_repr())

        return self.name_or_nothing() + " & ".join(names)


class DiscardLeftParser(Generic[Input, Output], Parser[Input, Output]):
    def __init__(self, left: Parser[Input, Any], right: Parser[Input, Output]):
        super().__init__()
        self.left = left
        self.right = right

    def _consume(self, state: State[Input], reader: Reader[Input]):
        status = self.left.consume(state, reader)
        if isinstance(status, Continue):
            return self.right.consume(state, status.remainder)
        else:
            return None

    def __repr__(self):
        string = f"{self.left.name_or_repr()} >> {self.right.name_or_repr()}"
        return self.name_or_nothing() + string


class DiscardRightParser(Generic[Input, Output], Parser[Input, Output]):
    def __init__(self, left: Parser[Input, Output], right: Parser[Input, Any]):
        super().__init__()
        self.left = left
        self.right = right

    def _consume(self, state: State[Input], reader: Reader[Input]):
        status1 = self.left.consume(state, reader)
        if isinstance(status1, Continue):
            status2 = self.right.consume(state, status1.remainder)
            if isinstance(status2, Continue):
                return Continue(status2.remainder, status1.value)
            else:
                return None
        else:
            return None

    def __repr__(self):
        string = f"{self.left.name_or_repr()} << {self.right.name_or_repr()}"
        return self.name_or_nothing() + string
